﻿var events = require('events');
var moment = require('moment');

// open connection to a serial port // create an empty modbus client 
var ModbusRTU = require("modbus-serial");
var ModbusClient = new ModbusRTU();

//---------------------------------------------
// Power Metering Definitions
//#region Power Metering Defintions

var PowerMeteringIo;// Holds socket io object
var PowerMeteringLogger;// Holds logger object

var PowerMeterTimeArray = [];

var PowerMeterArray = [
    { "Address" : "0", "Name" : "Meter 1", "MeterValueArray": [] , "CurrentValue": 0, "Enabled": "FALSE", "SendEvents": "FALSE", "TimeChanged": 0 },
    { "Address" : "1", "Name" : "Meter 2", "MeterValueArray": [] , "CurrentValue": 0, "Enabled": "FALSE", "SendEvents": "FALSE", "TimeChanged": 0 },
    { "Address" : "2", "Name" : "Meter 3", "MeterValueArray": [] , "CurrentValue": 0, "Enabled": "FALSE", "SendEvents": "FALSE", "TimeChanged": 0 },
    { "Address" : "3", "Name" : "Meter 4", "MeterValueArray": [] , "CurrentValue": 0, "Enabled": "FALSE", "SendEvents": "FALSE", "TimeChanged": 0 }
];

var PowerMeterMaximumLength = 60;// A minutes worth of data
var PowerMeterInitialValue = 40;// Initial value
var PowerMeterUpperValue = 60;// Initial value
var PowerMeterLowerValue = 20;// Initial value

//#endregion Power Metering Defintions

//#region Initialize Power Metering module

// Initialize 
exports.InitializePowerMetering = function (io, logger, callback) {
    
    // Set the io connection
    PowerMeteringIo = io;
    PowerMeteringLogger = logger;
    
    // Set the initial value for the power metering arrays (incl. time array)
    PowerMeterTimeArray[0] = moment().format('LTS');
    for (var meter = 0; meter < PowerMeterArray.length; meter++) {
        PowerMeterArray[meter].CurrentValue = Number(PowerMeterInitialValue).toFixed(2);// Start with arbitrary value
        PowerMeteringLogger.info('PowerMetering.InitializePowerMetering: PowerMeterArray[' + meter + '].CurrentValue=' + PowerMeterArray[meter].CurrentValue);
        PowerMeterArray[meter].MeterValueArray.push(PowerMeterArray[meter].CurrentValue);
    }
    
    ModbusClient.connectRTUBuffered("/dev/ttyUSB0", { baudRate: 19200, parity: "none", stopBits: 1 });
    ModbusClient.setID(1);

    // Generate new random values for each meter ...
    setInterval(function () {
        UpdatePowerMeterValues();
    }, 3000);
}

//#endregion Initialize Access Control Module

//#region Temporary

function UpdatePowerMeterValues() {
    
    // Update time array (x-axis), keeping only the maximum length of data
    PowerMeterTimeArray.push(moment().format('LTS'));
    if (PowerMeterTimeArray.length > PowerMeterMaximumLength) {
        PowerMeterTimeArray.shift();// eliminate oldest value
    }
    
    // Update the value of the first array using actual values
    ModbusClient.readHoldingRegisters(0x60E, 2, function (err, data) {
        if (err) {
            PowerMeteringLogger.error("Error in read: " + err);
        } else {
            var current = data.buffer.readFloatBE();
            PowerMeteringLogger.info("Metering.UpdatePowerMeterValues: Meter value" + current.toFixed(3) + " Amps");
            PowerMeterArray[0].CurrentValue = current;
            PowerMeterArray[0].MeterValueArray.push(PowerMeterArray[0].CurrentValue);// Add newest data
        }
    });
    // Only keep the n most values in the array
    if (PowerMeterArray[0].MeterValueArray.length > PowerMeterMaximumLength) {
        PowerMeterArray[0].MeterValueArray.shift();// Get rid of oldest data
    }
    

    // Generate new values for each meter, keeping only the maximum length of data
    for (var meter = 1; meter < PowerMeterArray.length; meter++) {
        
        // Random up/down value
        var blip = Math.random() - 0.5;// Changes by +/- 0.5
        
        // Update the current value    
        PowerMeterArray[meter].CurrentValue = Number(parseFloat(PowerMeterArray[meter].CurrentValue) + parseFloat(blip)).toFixed(2);
        //PowerMeteringLogger.info('PowerMetering.UpdatePowerMeterValues: NEW PowerMeterArray[' + meter + '].CurrentValue=' + PowerMeterArray[meter].CurrentValue);
        if (PowerMeterArray[meter].CurrentValue > PowerMeterUpperValue) PowerMeterArray[meter].CurrentValue = PowerMeterUpperValue;
        if (PowerMeterArray[meter].CurrentValue < PowerMeterLowerValue) PowerMeterArray[meter].CurrentValue = PowerMeterLowerValue;
        
        PowerMeterArray[meter].MeterValueArray.push(PowerMeterArray[meter].CurrentValue);// Add newest data
    
        // Only keep the n most values in the array
        if (PowerMeterArray[meter].MeterValueArray.length > PowerMeterMaximumLength) {
            PowerMeterArray[meter].MeterValueArray.shift();// Get rid of oldest data
        }

        // Now send the array for graphing
        var event = { "PowerMeterTimeArray": PowerMeterTimeArray, "PowerMeterArray": PowerMeterArray };
        PowerMeteringIo.sockets.emit("PowerMeter", event);
    }
}
//#endregion Temporary